from starkcore.utils.subresource import SubResource
from starkcore.utils.checks import check_datetime_or_date


class Subscription(SubResource):
    """# Subscription object
    Subscription is a recurring payment that can be used to charge a user periodically.
    ## Attributes (return-only):
    - amount [integer]: amount to be charged in cents. ex: 1000 = R$ 10.00
    - amount_min_limit [integer]: minimum amount limit for the subscription. ex: 500 = R$ 5.00
    - bacen_id [string]: BACEN (Brazilian Central Bank) identifier.
    - created [datetime.datetime]: creation datetime for the subscription. ex: datetime(2020, 3, 10)
    - description [string]: description of the subscription.
    - installment_end [datetime.datetime]: end datetime for the installments. ex: datetime(2020, 3, 10)
    - installment_start [datetime.datetime]: start datetime for the installments. ex: datetime(2020, 3, 10)
    - interval [string]: interval for the recurring charge. ex: "monthly"
    - pull_retry_limit [integer]: maximum number of retries for pulling the payment.
    - receiver_bank_code [string]: bank code of the receiver.
    - receiver_name [string]: name of the receiver.
    - receiver_tax_id [string]: tax ID of the receiver.
    - reference_code [string]: reference code for the subscription.
    - sender_final_name [string]: final sender name.
    - sender_final_tax_id [string]: final sender tax ID.
    - status [string]: current status of the subscription.
    - type [string]: type of the subscription.
    - updated [datetime.datetime]: last update datetime for the subscription. ex: datetime(2020, 3, 10)
    """

    def __init__(self, amount, amount_min_limit=None, bacen_id=None, created=None, description=None,
        installment_end=None, installment_start=None, interval=None, pull_retry_limit=None, receiver_bank_code=None,
        receiver_name=None, receiver_tax_id=None, reference_code=None, sender_final_name=None, sender_final_tax_id=None,
        status=None, type=None, updated=None
    ):
        self.amount = amount
        self.amount_min_limit = amount_min_limit
        self.bacen_id = bacen_id
        self.created = check_datetime_or_date(created)
        self.description = description
        self.installment_end = check_datetime_or_date(installment_end)
        self.installment_start = check_datetime_or_date(installment_start)
        self.interval = interval
        self.pull_retry_limit = pull_retry_limit
        self.receiver_bank_code = receiver_bank_code
        self.receiver_name = receiver_name
        self.receiver_tax_id = receiver_tax_id
        self.reference_code = reference_code
        self.sender_final_name = sender_final_name
        self.sender_final_tax_id = sender_final_tax_id
        self.status = status
        self.type = type
        self.updated = check_datetime_or_date(updated)


_resource = {"class": Subscription, "name": "Subscription"}



from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ..utils import rest


class CreditHolmes(Resource):
    """# CreditHolmes object
    CreditHolmes are used to obtain debt information on your customers.
    Before you create a CreditHolmes, make sure you have your customer's express
    authorization to verify their information in the Central Bank's SCR.
    When you initialize a CreditHolmes, the entity will not be automatically
    created in the Stark Infra API. The 'create' function sends the objects
    to the Stark Infra API and returns the list of created objects.
    ## Parameters (required):
    - tax_id [string]: customer's tax ID (CPF or CNPJ) for whom the credit operations will be verified. ex: "20.018.183/0001-80"
    Parameters (optional):
    - competence [string, default 'two months before current date']: competence month of the operation verification, format: "YYYY-MM". ex: "2021-04"
    - tags [list of strings, default []]: list of strings for reference when searching for CreditHolmes. ex: [credit", "operation"]
    Attributes (return-only):
    - id [string]: unique id returned when the CreditHolmes is created. ex: "5656565656565656"
    - result [dictionary]: result of the investigation after the case is solved.
    - status [string]: current status of the CreditHolmes. ex: "created", "failed", "success"
    - created [datetime.datetime]: creation datetime for the CreditHolmes. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the CreditHolmes. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, tax_id, competence, result=None, tags=None, id=None, status=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.tax_id = tax_id
        self.competence = competence
        self.status = status
        self.tags = tags
        self.result = result
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": CreditHolmes, "name": "CreditHolmes"}


def create(holmes, user=None):
    """# Create CreditHolmes
    Send a list of CreditHolmes objects for creation at the Stark Infra API
    ## Parameters (required):
    - holmes [list of CreditHolmes objects]: list of CreditHolmes objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of CreditHolmes objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=holmes, user=user)


def get(id, user=None):
    """# Retrieve a specific CreditHolmes
    Receive a single CreditHolmes object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - CreditHolmes object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, status=None, tags=None, ids=None, after=None, before=None, user=None):
    """# Retrieve CreditHolmes
    Receive a generator of CreditHolmes objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: "created", "failed", "success"
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of CreditHolmes objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, status=None, tags=None, ids=None, after=None, before=None, user=None):
    """# Retrieve paged CreditHolmes
    Receive a list of up to 100 CreditHolmes objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: "created", "failed", "success"
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of CreditHolmes objects with updated attributes
    - cursor to retrieve the next page of CreditHolmes objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )



from re import search
from setuptools import setup, find_packages

with open('README.md') as f:
    README = f.read()

with open('starkinfra/__init__.py') as f:
    version = search(r'version = \"(.*)\"', f.read()).group(1)

setup(
    name="starkinfra",
    packages=find_packages(exclude=["tests"]),
    include_package_data=True,
    description="SDK to facilitate Python integrations with Stark Infra",
    long_description=README,
    long_description_content_type="text/markdown",
    license="MIT License",
    url="https://github.com/starkinfra/sdk-python",
    author="Stark Infra",
    author_email="developers@starkbank.com",
    keywords=["stark infra", "starkinfra", "sdk", "open banking", "openbanking", "banking", "open", "stark"],
    version=version,
    install_requires=[
        "starkcore>=0.1.0",
    ],
)



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class IssuingBillingTransaction(Resource):
    """# IssuingBillingTransaction object
    Check out our API Documentation at https://starkinfra.com/docs/api#issuing-billing-transaction
    """

    def __init__(self, id=None, amount=None, invoice_id=None, installment=None, installment_count=None,
                 balance=None, holder_name=None, source=None, external_id=None, description=None, card_ending=None,
                 tax=None, rate=None, merchant_amount=None, merchant_currency_code=None, created=None):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.invoice_id = invoice_id
        self.installment = installment
        self.installment_count = installment_count
        self.balance = balance
        self.holder_name = holder_name
        self.source = source
        self.external_id = external_id
        self.description = description
        self.card_ending = card_ending
        self.tax = tax
        self.rate = rate
        self.merchant_amount = merchant_amount
        self.merchant_currency_code = merchant_currency_code
        self.created = check_datetime(created)


_resource = {"class": IssuingBillingTransaction, "name": "IssuingBillingTransaction"}


def query(limit=None, after=None, before=None, invoice_id=None, tags=None, user=None):
    """# IssuingBillingTransaction object
    Check out our API Documentation at https://starkinfra.com/docs/api#issuing-billing-transaction
    """
    return rest.get_stream(
        resource=_resource,
        after=check_datetime(after),
        before=check_datetime(before),
        invoice_id=invoice_id,
        tags=tags,
        limit=limit,
        user=user,
    )

def page(cursor=None, limit=None, after=None, before=None, invoice_id=None, tags=None, user=None):
    """# IssuingBillingTransaction object
    Check out our API Documentation at https://starkinfra.com/docs/api#issuing-billing-transaction
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        after=check_datetime(after),
        before=check_datetime(before),
        tags=tags,
        invoice_id=invoice_id,
        limit=limit,
        user=user,
    )



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class PixStatement(Resource):
    """# PixStatement object
    The PixStatement object stores information about all the transactions that
    happened on a specific day at your settlement account according to the Central Bank.
    It must be created by the user before it can be accessed.
    This feature is only available for direct participants.
    When you initialize a PixStatement, the entity will not be automatically
    created in the Stark Infra API. The 'create' function sends the objects
    to the Stark Infra API and returns the created object.
    ## Parameters (required):
    - after [datetime.date]: transactions that happened at this date are stored in the PixStatement, must be the same as before. ex: datetime.date(2020, 3, 10)
    - before [datetime.date]: transactions that happened at this date are stored in the PixStatement, must be the same as after. ex: datetime.date(2020, 3, 10)
    - type [string]: types of entities to include in statement. Options: ["interchange", "interchangeTotal", "transaction"]
    ## Attributes (return-only):
    - id [string]: unique id returned when the PixStatement is created. ex: "5656565656565656"
    - status [string]: current PixStatement status. ex: "success" or "failed"
    - transaction_count [integer]: number of transactions that happened during the day that the PixStatement was requested. ex: 11
    - created [datetime.datetime]: creation datetime for the PixStatement. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the PixStatement. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, after, before, type, id=None, status=None, transaction_count=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.after = check_date(after)
        self.before = check_date(before)
        self.type = type
        self.status = status
        self.transaction_count = transaction_count
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": PixStatement, "name": "PixStatement"}


def create(statement, user=None):
    """# Create a PixStatement object
    Create a PixStatement linked to your Workspace in the Stark Infra API
    ## Parameters (optional):
    - statement [PixStatement object]: PixStatement object to be created in the API.
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - PixStatement object with updated attributes.
    """
    return rest.post_single(resource=_resource, entity=statement, user=user)


def get(id, user=None):
    """# Retrieve a PixStatement object
    Retrieve the PixStatement object linked to your Workspace in the Stark Infra API by its id.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - PixStatement object that corresponds to the given id.
    """
    return rest.get_id(id=id, resource=_resource, user=user)


def query(limit=None, ids=None, user=None):
    """# Retrieve PixStatements
    Receive a generator of PixStatement objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of PixStatement objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, ids=None, user=None):
    """# Retrieve paged PixStatements
    Receive a list of up to 100 PixStatement objects previously created in the Stark Infra API
    Use this function instead of query if you want to manually page your statements.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of PixStatement objects with updated attributes
    - cursor to retrieve the next page of PixStatement objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        ids=ids,
        user=user,
    )


def csv(id, user=None):
    """# Retrieve a .csv PixStatement
    Retrieve a specific PixStatement by its id in a .csv file.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - .zip file containing a PixStatement in .csv format
    """
    return rest.get_content(resource=_resource, id=id, user=user, sub_resource_name="csv")



from starkcore.utils.api import from_api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ...utils import rest
from ..__issuingtoken import _resource as _issuing_token_resource
from starkcore.error import Error


class Log(Resource):
    """# issuingtoken.Log object
    Every time an IssuingToken entity is modified, a corresponding issuingtoken.Log
    is generated for the entity. This log is never generated by the user.
    ## Attributes (return-only):
    - id [string]: unique id returned when the log is created. ex: "5656565656565656"
    - token [IssuingToken]: IssuingToken entity to which the log refers to.
    - errors [list of strings]: list of errors linked to this IssuingToken event.
    - type [string]: type of the IssuingToken event which triggered the log creation. ex: "active", "blocked", "canceled", "frozen" or "pending"
    - created [datetime.datetime]: creation datetime for the log. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id, type, errors, token, created):
        Resource.__init__(self, id=id)

        self.token = from_api_json(_issuing_token_resource, token)
        self.type = type
        self.created = created
        self.errors = errors


_resource = {"class": Log, "name": "IssuingTokenLog"}


def get(id, user=None):
    """# Retrieve a specific issuingtoken.Log
    Receive a single issuingtoken.Log object previously created by the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - issuingtoken.Log object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, types=None, token_ids=None, ids=None, user=None):
    """# Retrieve issuingtoken.Log
    Receive a generator of issuingtoken.Log objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter for log event types. ex: ["active", "blocked", "canceled", "frozen", "pending"]
    - token_ids [list of strings, default None]: list of IssuingToken ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - generator of issuingtoken.Log objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        token_ids=token_ids,
        ids=ids,
        user=user,
    )


def page(limit=None, after=None, before=None, types=None, token_ids=None, cursor = None, ids=None, user=None):
    """# Retrieve paged issuingtoken.Log
    Receive a list of up to 100 issuingtoken.Log objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter for log event types. ex: ["active", "blocked", "canceled", "frozen", "pending"]
    - token_ids [list of strings, default None]: list of IssuingToken ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of issuingtoken.Log objects with updated attributes
    - cursor to retrieve the next page of issuingtoken.Log objects
    """
    return rest.get_page(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        token_ids=token_ids,
        cursor=cursor,
        ids=ids,
        user=user,
    )



from starkcore.utils.resource import Resource
from ..utils import rest


class IssuingBalance(Resource):
    """# IssuingBalance object
    The IssuingBalance object displays the current issuing balance of the Workspace,
    which is the result of the sum of all transactions within this
    Workspace. The balance is never generated by the user, but it
    can be retrieved to see the available information.
    ## Attributes (return-only):
    - id [string]: unique id returned when IssuingBalance is created. ex: "5656565656565656"
    - amount [integer]: current balance amount of the Workspace in cents. ex: 200 (= R$ 2.00)
    - currency [string]: currency of the current Workspace. Expect others to be added eventually. ex: "BRL"
    - updated [datetime.datetime]: latest update datetime for the IssuingBalance. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id, amount, currency, updated):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.currency = currency
        self.updated = updated


_resource = {"class": IssuingBalance, "name": "IssuingBalance"}


def get(user=None):
    """# Retrieve the IssuingBalance object
    Receive the IssuingBalance object linked to your Workspace in the Stark Infra API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - IssuingBalance object with updated attributes
    """
    return next(rest.get_stream(resource=_resource, user=user))



from random import choice
from datetime import datetime


_randomSource = [c for c in "abcdefghijklmnopqrstuvwxyz"]
_randomSource += [c.upper() for c in _randomSource]
_randomSource += [c for c in "0123456789"]


def create(bank_code):
    return "{bank_code}{date}{randomString}".format(
        bank_code=bank_code,
        date=datetime.utcnow().strftime("%Y%m%d%H%M"),
        randomString=''.join(choice(_randomSource) for _ in range(11)),
    )



from base64 import b64encode
from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class IndividualDocument(Resource):
    """# IndividualDocument object
    Individual documents are images containing either side of a document or a selfie
    to be used in a matching validation. When created, they must be attached to an individual
    identity to be used for its validation.
    When you initialize a IndividualDocument, the entity will not be automatically
    created in the Stark Infra API. The 'create' function sends the objects
    to the Stark Infra API and returns the list of created objects.
    ## Parameters (required):
    - type [string]: type of the IndividualDocument. Options: "drivers-license-front", "drivers-license-back", "identity-front", "identity-back" or "selfie"
    - content [string]: Base64 data url of the picture. ex: data:image/jpeg;base64,/9j/4AAQSkZJRgABAQAASABIAAD...
    - content_type [string]: content MIME type. This parameter is required as input only. ex: "image/png" or "image/jpeg"
    - identity_id [string]: unique id of IndividualIdentity. ex: "5656565656565656"
    ## Parameters (optional):
    - tags [list of strings, default []]: list of strings for reference when searching for IndividualDocuments. ex: ["employees", "monthly"]
    ## Attributes (return-only):
    - id [string]: unique id returned when the IndividualDocument is created. ex: "5656565656565656"
    - status [string]: current status of the IndividualDocument. Options: "created", "canceled", "processing", "failed", "success"
    - created [datetime.datetime]: creation datetime for the IndividualDocument. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, type, content, identity_id, content_type=None, tags=None, id=None, status=None, created=None):
        Resource.__init__(self, id=id)
        self.type = type
        self.identity_id = identity_id
        self.tags = tags
        self.status = status
        self.created = check_datetime(created)
        self.content = content
        self.content_type = content_type

        if content_type:
            self.content = "data:{content_type};base64,{content}".format(
                content_type=content_type,
                content=b64encode(content).decode('utf-8')
            )
            self.content_type = None


_resource = {"class": IndividualDocument, "name": "IndividualDocument"}


def create(documents, user=None):
    """# Create IndividualDocuments
    Send a list of IndividualDocument objects for creation at the Stark Infra API
    ## Parameters (required):
    - documents [list of IndividualDocument objects]: list of IndividualDocument objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of IndividualDocument objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=documents, user=user)


def get(id, user=None):
    """# Retrieve a specific IndividualDocument
    Receive a single IndividualDocument object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - IndividualDocument object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, status=None, tags=None, ids=None, after=None, before=None, user=None):
    """# Retrieve IndividualDocuments
    Receive a generator of IndividualDocument objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. Options: ["created", "canceled", "processing", "failed", "success"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of IndividualDocument objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, status=None, tags=None, ids=None, after=None, before=None, user=None):
    """# Retrieve paged IndividualDocuments
    Receive a list of up to 100 IndividualDocument objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. Options: ["created", "canceled", "processing", "failed", "success"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of IndividualDocument objects with updated attributes
    - cursor to retrieve the next page of IndividualDocument objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )

